package gearmanij.client;

import java.nio.ByteBuffer;
import java.util.LinkedList;
import java.util.Queue;
import java.util.UUID;

import gearmanij.GearmanException;
import gearmanij.Packet;
import gearmanij.PacketType;

//TODO take a long hard at the drive task state machine, seems unfinished to me
//TODO add a meaning description method that can be to describe the task when logging
//TODO implement EXCEPTION case in driveTask state machine
class GearmanServerRequest {

  
  enum RequestState {
    NEW, SUBMITTING, AWAITINGRESPONSE, PROCESSINGRESPONSES, FINISHED, EXCEPTION
  }
  
  private static final String DESCRIPTION_PREFIX = "GearmanServerRequest";
  private final String DESCRIPTION;
  private RequestState state = null;
  private Packet requestPacket = null;
  private Queue<Packet >responsePackets = new LinkedList<Packet>();
  private GearmanJobServerSession session = null;
  private UUID uuid = null;
  private GearmanServerResponseHandler handler = null;
  
  GearmanServerRequest(GearmanServerResponseHandler job, Packet request) {
    state = RequestState.NEW;
    uuid = UUID.randomUUID();
    this.handler = job;
    requestPacket = request; //TODO make a copy of the packet or make sure the packet is hardened against tampering
    DESCRIPTION = new String(DESCRIPTION_PREFIX + ":" + job + ":" + uuid);
  }
  
  RequestState getState() {
    return state;
  }
  
  void setSession (GearmanJobServerSession session) {
    //TODO protect against reset
    assert (session == null);
    assert (state == RequestState.NEW);
    this.session = session;
    changeState(RequestState.SUBMITTING);
  }
  
  ByteBuffer getRequestAsByteBuffer() {
    //TODO check that I've got one
    return requestPacket.toByteBuffer();
  }
  
  void addResponsePacket(Packet p) {
    synchronized (responsePackets) {
      responsePackets.add(p);      
    }
    //TODO validate this a valid wellformed packet
  }
  
  GearmanServerResponseHandler getHandler() {
    return handler;
  }
  
  public String toString () {
    return DESCRIPTION;
  }
  
  void driveRequest() throws GearmanException { 
    Packet p = null; 
    while (true) {
      switch (state) {
      
      // Task needs to be submitted to server
      case SUBMITTING:
        if (session == null) {
          changeState(RequestState.EXCEPTION);
          break;
        }
        if (requestPacket == null) {
          changeState(RequestState.EXCEPTION);
          break;
        }
        session.sendTaskToServer(this);
      
       //Task has been submitted to server but has not
       //has not received a response
      case AWAITINGRESPONSE:
        synchronized (responsePackets) {
          if (responsePackets.isEmpty()) {
            changeState(RequestState.AWAITINGRESPONSE);
            return;
          }      
          p = responsePackets.peek();
        }
        if (PacketType.isJobSubmission(requestPacket.getPacketType()) && 
            p.getPacketType().equals(PacketType.JOB_CREATED)) {
          changeState(RequestState.PROCESSINGRESPONSES);
          break;
        } else if (requestPacket.getPacketType().equals(PacketType.OPTION_REQ) && 
            p.getPacketType().equals(PacketType.OPTION_RES)) {
          changeState(RequestState.PROCESSINGRESPONSES);
          break;
        } else if (requestPacket.getPacketType().equals(PacketType.ECHO_REQ) && 
            p.getPacketType().equals(PacketType.ECHO_RES)) {
          changeState(RequestState.PROCESSINGRESPONSES);
          break;
        } else if (p.getPacketType().equals(PacketType.STATUS_RES)) {
          changeState(RequestState.PROCESSINGRESPONSES);
          break;
        } else if (p.getPacketType().equals(PacketType.ERROR)) {
          changeState(RequestState.PROCESSINGRESPONSES);
          break;
        } else {
          changeState(RequestState.EXCEPTION);
          break;
        }
        
      case PROCESSINGRESPONSES:
        synchronized (responsePackets) {
          if (responsePackets.isEmpty()) {
            changeState(RequestState.PROCESSINGRESPONSES);
            return;
          }
          p = responsePackets.remove();
        }
        try {
          handler.handleResponse(p);
          if (!handler.responseComplete()) {
            changeState(RequestState.PROCESSINGRESPONSES);
            return;  
          }
        } catch (GearmanException ge) {
          changeState(RequestState.EXCEPTION);
          break;
        }
        changeState(RequestState.FINISHED);
        
      case FINISHED:
        return;
      
      case EXCEPTION:
        throw new GearmanException("Encountered Fatal Exception while driving the following ServerRequest: " + requestPacket); 
      
      default:         
      }
    }      
  }
  
  private void changeState(RequestState newState) {
    if (! newState.equals(state)) {
      this.state = newState;
    }
  }

}


