package gearmanij.client;

import gearmanij.Connection;
import gearmanij.Constants;
import gearmanij.GearmanException;
import gearmanij.PacketType;
import gearmanij.SimpleWorker;
import gearmanij.SocketConnection;
import gearmanij.Worker;
import gearmanij.example.ReverseFunction;
import gearmanij.util.ByteUtils;

import java.io.IOException;
import java.util.Collection;
import java.util.Map;

import junit.framework.Assert;

import org.junit.Before;
import org.junit.Test;


//TODO ... well for starters ... finish me

public class GearmanClientJobTest {
  

  GearmanClient gc = new GearmanClient();
  @Before
  public void initClient() throws IOException, GearmanException {
    gc.addJobServer("127.0.0.1", Constants.GEARMAN_DEFAULT_TCP_PORT);
  }
  
  @Before
  public void initWorker () {
    Thread workerThread = startThread("Worker", new Runnable() {
      public void run() {
        // worker connect to server
        SocketConnection conn = new SocketConnection();
        conn.setLog(System.err);

        Worker reverse = new SimpleWorker();
        reverse.addServer(conn);
        reverse.registerFunction(new ReverseFunction());

        for (int i = 0; i < 100; i++) {
          //System.out.println("Loop Number: " + i);
          Map<Connection, PacketType> jobs = reverse.grabJob();
          PacketType packetType = jobs.get(conn);

          if (packetType == PacketType.NO_JOB) {
            try {
              Thread.sleep(100);
              } catch (InterruptedException ioe) {};
          } else if (packetType == PacketType.JOB_ASSIGN) {
            System.out.println("RECEIVED JOB FOR ASSIGMENT");
          } else if (packetType == PacketType.NOOP) {
            //System.out.println("noop");
          } else {
            //System.out.println("YIKES!");
          }
        }
        System.out.println("FINISHED");
      }
  });
  }
  
  private static Thread startThread(String threadName, Runnable target) {
    Thread t = new Thread(target, threadName);
    System.out.println("Starting " + threadName);
    t.start();
    try {
    Thread.sleep(100);
    } catch (InterruptedException ioe) {};
    return t;
  }
    
  @Test
  public void reverseTest () throws IOException, GearmanException {
    for (int i = 0; i < 10; i++) {
      boolean jobCreatedEvent = false;
      boolean jobCompletedEvent = false;
      StringBuffer text = generateData(8193, "Hello World");
      GearmanJob job = new GearmanJob("reverse", ByteUtils.toAsciiBytes(text
          .toString()), false, GearmanJob.JobPriority.NORMAL);
      gc.doJob(job);
      while (true) {
        if (jobCreatedEvent && jobCompletedEvent) {
          break;
        }
        Collection <GearmanJob>updated = gc.selectUpdatedJobs();
        if (!updated.contains(job))
          if (job.getStatus().equals(GearmanJob.Status.INPROGRESS))
            jobCreatedEvent = true;
        if (job.getStatus().equals(GearmanJob.Status.COMPLETED))
          jobCompletedEvent = true;
        
      }
      String resultString = ByteUtils.fromAsciiBytes(job.getResults());
      Assert.assertTrue(resultString.equals(text.reverse().toString()));
    }
  }

  @Test
  public void reverseTestBG () throws IOException, GearmanException {
    StringBuffer text = new StringBuffer("Hello World!");
    GearmanJob job = new GearmanJob("reverse",ByteUtils.toAsciiBytes(text.toString()),true, GearmanJob.JobPriority.NORMAL);
    gc.doJob(job);
    Collection<GearmanJob> updateJobs = null;
    GearmanJobStatus status = null;
    while (!job.isCompleted())
      gc.selectUpdatedJobs();
    do {
      status = gc.updateJobStatus(job);
    } while (status.isKnown());
  }
   
  private StringBuffer generateData(int size, String pattern) {
    StringBuffer b = new StringBuffer();
    int leftToWrite = size;
    while (leftToWrite > 0) {
      int amount = pattern.length() > leftToWrite ? leftToWrite : pattern
          .length();
      b.append(pattern.substring(0, amount));
      leftToWrite -= amount;
    }
    return b;
  }


}
